import { AxiosRequestConfig } from 'axios';
import { LaunchOptions } from 'puppeteer';

declare enum EngineType {
    Html = "html",
    Spa = "spa",
    Json = "json",
    Xml = "xml"
}
type EngineTypes = "html" | "spa" | "json" | "xml";
type TScrapedData = Record<string, any>;
type TScrapedDataPromise = Promise<TScrapedData>;
interface ISelector {
    selector: string;
    attribute?: string;
}
type TSelectors = Record<string, ISelector>;
interface IScrapeHtmlOptions {
    selectors: TSelectors;
    request?: AxiosRequestConfig;
    plugins?: ITransformerPlugin[];
}
interface IScrapeSpaOptions {
    selectors: TSelectors;
    request?: LaunchOptions;
    plugins?: ITransformerPlugin[];
}
interface IScrapeJsonOptions {
    selectors: TSelectors;
    request?: AxiosRequestConfig;
    plugins?: ITransformerPlugin[];
}
interface IScrapeXmlOptions {
    selectors: TSelectors;
    request?: AxiosRequestConfig;
    plugins?: ITransformerPlugin[];
}
type IScrapeDefaultOptions = IScrapeHtmlOptions & IScrapeSpaOptions & IScrapeJsonOptions & IScrapeXmlOptions;
interface IScrapeOptions {
    url: string;
    engine: EngineType | EngineTypes;
    options: IScrapeDefaultOptions;
}
declare enum PluginType {
    Transformer = "transformer",
    Engine = "engine"
}
type PluginTypes = "transformer" | "engine";
interface IPlugin {
    pluginType: string;
}
interface ITransformerPlugin extends IPlugin {
    meta?: any;
    supportedEngines?: string[];
    preProcess?: (html: string) => string;
    postProcess?: (data: TScrapedData) => TScrapedData;
    initialize?: (options: {
        selectors: TSelectors;
    }) => void;
}
interface IEnginePlugin extends IPlugin {
    engine: string;
    scrape: (url: string, options: IScrapeDefaultOptions) => Promise<TScrapedData>;
}

declare const scrape: ({ url, engine, options, }: IScrapeOptions) => TScrapedDataPromise;

declare const scrapeHtml: (url: string, { selectors, request, plugins }: IScrapeHtmlOptions) => TScrapedDataPromise;

declare const scrapeSpa: (url: string, { selectors, request, plugins }: IScrapeSpaOptions) => TScrapedDataPromise;

declare const scrapeXml: (url: string, { selectors, request, plugins }: IScrapeXmlOptions) => TScrapedDataPromise;

declare const scrapeJson: (url: string, { selectors, request, plugins }: IScrapeXmlOptions) => TScrapedDataPromise;

declare enum ConditionOperationType {
    Difference = "difference",
    Equal = "equal",
    Contains = "contains",
    Regex = "regex"
}
type ConditionOperationTypes = "difference" | "equal" | "contains" | "regex";
interface ConditionType {
    value: string;
    operation: ConditionOperationType | ConditionOperationTypes;
    sensitive?: boolean;
}
interface ConditionPluginType extends Record<string, ConditionType[]> {
}

declare const plugin: (conditions: ConditionPluginType) => ITransformerPlugin;

export { ConditionOperationType, ConditionType, EngineType, EngineTypes, IEnginePlugin, IPlugin, IScrapeDefaultOptions, IScrapeHtmlOptions, IScrapeJsonOptions, IScrapeOptions, IScrapeSpaOptions, IScrapeXmlOptions, ISelector, ITransformerPlugin, PluginType, PluginTypes, TScrapedData, TScrapedDataPromise, TSelectors, plugin as conditionsPlugin, scrape, scrapeHtml, scrapeJson, scrapeSpa, scrapeXml };
